@@ -1,13 +1,3 @@
-#include <sys/types.h>
-
-#include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <assert.h>
-
 #include "buffer.h"
 #include "server.h"
 #include "keyvalue.h"
@@ -24,6 +14,16 @@
 #include "inet_ntop_cache.h"
 #include "crc32.h"
 
+#include <sys/types.h>
+
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <assert.h>
+
 #include <stdio.h>
 
 #ifdef HAVE_SYS_FILIO_H
@@ -756,12 +756,15 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 
 	switch(hctx->state) {
 	case PROXY_STATE_INIT:
+#if defined(HAVE_IPV6) && defined(HAVE_INET_PTON)
 		if (strstr(host->host->ptr,":")) {
 		    if (-1 == (hctx->fd = socket(AF_INET6, SOCK_STREAM, 0))) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed: ", strerror(errno));
 			return HANDLER_ERROR;
 		    }
-		} else {
+		} else
+#endif
+		{
 		    if (-1 == (hctx->fd = socket(AF_INET, SOCK_STREAM, 0))) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed: ", strerror(errno));
 			return HANDLER_ERROR;
@@ -842,17 +845,14 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 
 		chunkqueue_remove_finished_chunks(hctx->wb);
 
-		if (-1 == ret) {
-			if (errno != EAGAIN &&
-			    errno != EINTR) {
-				log_error_write(srv, __FILE__, __LINE__, "ssd", "write failed:", strerror(errno), errno);
+		if (-1 == ret) { /* error on our side */
+			log_error_write(srv, __FILE__, __LINE__, "ssd", "write failed:", strerror(errno), errno);
 
-				return HANDLER_ERROR;
-			} else {
-				fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
+			return HANDLER_WAIT_FOR_EVENT;
+		} else if (-2 == ret) { /* remote close */
+			log_error_write(srv, __FILE__, __LINE__, "ssd", "write failed, remote connection close:", strerror(errno), errno);
 
-				return HANDLER_WAIT_FOR_EVENT;
-			}
+			return HANDLER_WAIT_FOR_EVENT;
 		}
 
 		if (hctx->wb->bytes_out == hctx->wb->bytes_in) {
@@ -1047,11 +1047,32 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 			 *
 			 */
 
-			proxy_connection_close(srv, hctx);
-			joblist_append(srv, con);
+			if (hctx->host) {
+				hctx->host->is_disabled = 1;
+				hctx->host->disable_ts = srv->cur_ts;
+				log_error_write(srv, __FILE__, __LINE__,  "sbdd", "proxy-server disabled:",
+						hctx->host->host,
+						hctx->host->port,
+						hctx->fd);
 
-			con->http_status = 503;
-			con->mode = DIRECT;
+				/* disable this server */
+				hctx->host->is_disabled = 1;
+				hctx->host->disable_ts = srv->cur_ts;
+
+				proxy_connection_close(srv, hctx);
+
+				/* reset the enviroment and restart the sub-request */
+				buffer_reset(con->physical.path);
+				con->mode = DIRECT;
+
+				joblist_append(srv, con);
+			} else {
+				proxy_connection_close(srv, hctx);
+				joblist_append(srv, con);
+
+				con->mode = DIRECT;
+				con->http_status = 503;
+			}
 
 			return HANDLER_FINISHED;
 		}
@@ -1107,19 +1128,20 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 
 	/* check if extension matches */
 	for (k = 0; k < p->conf.extensions->used; k++) {
+		data_array *ext = NULL;
 		size_t ct_len;
 
-		extension = (data_array *)p->conf.extensions->data[k];
+		ext = (data_array *)p->conf.extensions->data[k];
 
-		if (extension->key->used == 0) continue;
+		if (ext->key->used == 0) continue;
 
-		ct_len = extension->key->used - 1;
+		ct_len = ext->key->used - 1;
 
 		if (s_len < ct_len) continue;
 
 		/* check extension in the form "/proxy_pattern" */
-		if (*(extension->key->ptr) == '/') {
-			if (strncmp(fn->ptr, extension->key->ptr, ct_len) == 0) {
+		if (*(ext->key->ptr) == '/') {
+			if (strncmp(fn->ptr, ext->key->ptr, ct_len) == 0) {
 				if (s_len > ct_len + 1) {
 					char *pi_offset;
 
@@ -1127,15 +1149,17 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 						path_info_offset = pi_offset - fn->ptr;
 					}
 				}
+				extension = ext;
 				break;
 			}
-		} else if (0 == strncmp(fn->ptr + s_len - ct_len, extension->key->ptr, ct_len)) {
+		} else if (0 == strncmp(fn->ptr + s_len - ct_len, ext->key->ptr, ct_len)) {
 			/* check extension in the form ".fcg" */
+			extension = ext;
 			break;
 		}
 	}
 
-	if (k == p->conf.extensions->used) {
+	if (NULL == extension) {
 		return HANDLER_GO_ON;
 	}
 
